import * as ini from "ini";
import * as  fs from "fs";

type ConfigResultType = { [key: string]: { [key: string]: string } };

interface PropertyConfig {
   required?: boolean;
   default?: string;
}

type ConfigType<T extends ConfigResultType> = {
   [k in keyof T]: {
      [k2 in keyof T[k]]: PropertyConfig;
   }
}

export default function config<T extends ConfigResultType>(config: ConfigType<T>, file?: string): T {
   let res = {} as T;
   const parsed = (file && fs.existsSync(file)) ? ini.parse(fs.readFileSync(file).toString("utf8")) : {};
   let errors = [];
   for (let C in config) {
      let category = {} as any;
      for (let F in config[C]) {
         const envname = `${C.toUpperCase()}_${F.toUpperCase()}`;
         const value = (parsed[C] || {})[F] || process.env[envname] || config[C][F].default;
         if (!value && config[C][F].required)
            errors.push(`Field ${C}->${F} is required. Set in in ${file} or as environment variable ${envname}.`);
         else
            category[F] = value;
      }
      res[C] = category;
   }

   if (errors.length > 0) {
      throw new Error("There are errors: \n" + errors.join("\n"))
   }

   return res;
}

